package com.example.demo.config;

import javax.annotation.PostConstruct;

import org.springframework.amqp.core.AmqpAdmin;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.ExchangeBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.QueueBuilder;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

//@Configuration
public class DirectExchangeConfig {

	@Value("${rabbitmq.direct.queueName1}")
	private String directQueue1;

	@Value("${rabbitmq.direct.queueName2}")
	private String directQueue2;

	@Value("${rabbitmq.direct.exchange}")
	private String directExchange;

	@Value("${rabbitmq.direct.routingKey1}")
	private String routingKey1;

	@Value("${rabbitmq.direct.routingKey2}")
	private String routingKey2;

	@Autowired
	private AmqpAdmin ampAdmin;

	@PostConstruct
	public void init() {
		ampAdmin.declareQueue(queue1());
		ampAdmin.declareQueue(queue2());
		ampAdmin.declareExchange(exchange());
		ampAdmin.declareBinding(binding1());
		ampAdmin.declareBinding(binding2());

	}

	public Queue queue1() {
		
		return QueueBuilder
				.durable(directQueue1)
				.build();
		
	}

	public Queue queue2() {

		return QueueBuilder.durable(directQueue2).build();
	}

	public DirectExchange exchange() {

		return ExchangeBuilder.directExchange(directExchange).build();

	}

	public Binding binding1() {

		return BindingBuilder.bind(queue1()).to(exchange()).with(routingKey1);
	}

	public Binding binding2() {

		return BindingBuilder.bind(queue2()).to(exchange()).with(routingKey2);
	}
	
	@Bean
	public AmqpTemplate amqpTemplate(ConnectionFactory connectionFactory) {
		
		RabbitTemplate rabbitTemplate = new RabbitTemplate();
		rabbitTemplate.setConnectionFactory(connectionFactory);
		rabbitTemplate.setExchange(directExchange);
		
		return rabbitTemplate;
		
	}

}
