package com.example.demo.config;

import javax.annotation.PostConstruct;

import org.springframework.amqp.core.AmqpAdmin;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.ExchangeBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.QueueBuilder;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;


@Configuration
public class TopicExConfig {

	@Value("${rabbit.topic.scienceQueue}")
	private String scienceQueue;
	@Value("${rabbit.topic.artQueue}")
	private String artsQueue;
	@Value("${rabbit.topic.computerQueue}")
	private String computerQueue;

	@Value("${rabbitmq.topic.exchange}")
	private String exchange;

	@Value("${rabbitmq.topic.rkSci}")
	private String scieceRk;

	@Value("${rabbitmq.topic.rkArts}")
	private String artsRK;

	@Value("${rabbitmq.topic.rkComputer}")
	private String computerRk;

	@Autowired
	private AmqpAdmin amqpAdmin;

	@PostConstruct
	public void init() {

		amqpAdmin.declareQueue(scienceQueue());
		amqpAdmin.declareQueue(computerQueue());
		amqpAdmin.declareQueue(artsQueue());
		
		amqpAdmin.declareExchange(college());
		
		amqpAdmin.declareBinding(bindingWithArtsQueue());
		amqpAdmin.declareBinding(bindingWithSciQueue());
		amqpAdmin.declareBinding(bindingWithComputerQueue());
	}

	// define a queue

	public Queue scienceQueue() {

		return QueueBuilder.durable(scienceQueue).build();
	}

	public Queue computerQueue() {

		return QueueBuilder.durable(computerQueue).build();
	}

	public Queue artsQueue() {

		return QueueBuilder.durable(artsQueue).build();
	}

	public TopicExchange college() {

		return ExchangeBuilder.topicExchange(exchange).build();
	}

	public Binding bindingWithSciQueue() {

		return BindingBuilder.bind(scienceQueue()).to(college()).with(scieceRk);
	}

	public Binding bindingWithArtsQueue() {

		return BindingBuilder.bind(artsQueue()).to(college()).with(artsRK);
	}
	
	public Binding bindingWithComputerQueue() {

		return BindingBuilder.bind(computerQueue()).to(college()).with(computerRk);
	}
	
	
	@Bean
	public AmqpTemplate amqpTemplate(ConnectionFactory connectionFactory) {
		
		RabbitTemplate rabbitTemplate = new RabbitTemplate();
		rabbitTemplate.setConnectionFactory(connectionFactory);
		rabbitTemplate.setExchange(exchange);
		
		
		return rabbitTemplate;
		
	}
	
	
	
	
	
	
	
	
	

}
